from datetime import datetime
from unittest.mock import AsyncMock, patch

import pytest
from openg2p_g2p_bridge_api.controllers import DisbursementStatusController
from openg2p_g2p_bridge_models.errors.codes import G2PBridgeErrorCodes
from openg2p_g2p_bridge_models.errors.exceptions import DisbursementStatusException
from openg2p_g2p_bridge_models.schemas import (
    DisbursementStatusPayload,
    DisbursementStatusRequest,
    DisbursementStatusResponse,
)
from openg2p_g2pconnect_common_lib.schemas import (
    RequestHeader,
    StatusEnum,
    SyncResponseHeader,
)


@pytest.mark.asyncio
@patch("openg2p_g2p_bridge_api.services.DisbursementStatusService.get_component")
async def test_get_disbursement_status_success(mock_service_get_component):
    # Setup mock service
    mock_service_instance = AsyncMock()
    mock_service_get_component.return_value = mock_service_instance

    # Mock service methods
    mock_service_instance.get_disbursement_status_payloads = AsyncMock(
        return_value=[
            DisbursementStatusPayload(
                disbursement_id="disb123",
                disbursement_recon_records=None,
            )
        ]
    )

    expected_response = DisbursementStatusResponse(
        header=SyncResponseHeader(
            message_id="",
            message_ts=datetime.now().isoformat(),
            action="",
            status=StatusEnum.succ,
            status_reason_message="",
        ),
        message=[
            DisbursementStatusPayload(
                disbursement_id="disb123",
                disbursement_recon_records=None,
            )
        ],
    )

    mock_service_instance.construct_disbursement_status_success_response = AsyncMock(
        return_value=expected_response
    )

    # Instantiate controller and make request
    controller = DisbursementStatusController()
    request_payload = DisbursementStatusRequest(
        header=RequestHeader(
            message_id="123",
            message_ts=datetime.now().isoformat(),
            action="",
            sender_id="",
            sender_uri="",
            receiver_id="",
            total_count=1,
            is_msg_encrypted=False,
        ),
        message=["disb123"],
    )

    actual_response = await controller.get_disbursement_status(request_payload)
    assert actual_response == expected_response


@pytest.mark.asyncio
@patch("openg2p_g2p_bridge_api.services.DisbursementStatusService.get_component")
@pytest.mark.parametrize("error_code", list(G2PBridgeErrorCodes))
async def test_get_disbursement_status_failure(mock_service_get_component, error_code):
    # Setup mock service
    mock_service_instance = AsyncMock()
    mock_service_get_component.return_value = mock_service_instance

    # Mock the method to raise an error
    mock_service_instance.get_disbursement_status_payloads.side_effect = (
        DisbursementStatusException(code=error_code.value, message=error_code.value)
    )

    error_response = DisbursementStatusResponse(
        header=SyncResponseHeader(
            message_id="",
            message_ts=datetime.now().isoformat(),
            action="",
            status=StatusEnum.rjct,
            status_reason_message=error_code.value,
        ),
        message=[],
    )

    mock_service_instance.construct_disbursement_status_error_response = AsyncMock(
        return_value=error_response
    )

    # Instantiate controller and make request
    controller = DisbursementStatusController()
    request_payload = DisbursementStatusRequest(
        header=RequestHeader(
            message_id="123",
            message_ts=datetime.now().isoformat(),
            action="",
            sender_id="",
            sender_uri="",
            receiver_id="",
            total_count=1,
            is_msg_encrypted=False,
        ),
        message=["disb123"],
    )

    # Try to get disbursement status and catch any raised exception
    try:
        actual_response = await controller.get_disbursement_status(request_payload)
    except DisbursementStatusException:
        # If an exception is raised, assert that it matches the expected mock response
        actual_response = (
            await mock_service_instance.construct_disbursement_status_error_response(
                request_payload
            )
        )

    # Assert individual fields to handle mock object comparison issues
    assert actual_response.header.status == error_response.header.status
    assert (
        actual_response.header.status_reason_message
        == error_response.header.status_reason_message
    )
    assert actual_response.message == error_response.message

    # Assert overall response equality
    assert (
        actual_response == error_response
    ), f"The response did not match the expected error response for {error_code}."
